/*
 * Copyright (C) 2001  Jonathan T. Ledlie
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef CFS_H
#define CFS_H

#include "md5.h"
#include "disk.h"
#include "group.h"
#include "status.h"
#include "bufferManager.h"

struct QuickDigest {
  md5_byte_t digest[DIGEST_LENGTH];
  //  string url;
  char *url;
  QuickDigest *next;
  QuickDigest (const char *url, md5_byte_t *digest);
  ~QuickDigest ();
};

class CFS {
  friend ostream& operator << (ostream& os, CFS * &cfs);

 public:
  CFS (const char *const rawDiskDevice, int groupCount, int frameCount, Status& returnStatus);
  ~CFS ();

  const Status lookup (const char *const url, int & groupIndex);
  
  const Status create (const char *const url, const int groupIndex, char * data, int size);
  const Status read (const char *const url, const int groupIndex, char *&data, int &length);
  const Status release (const char *const url, const int groupIndex);
  const Status remove (const char *const url, const int groupIndex);

  const Status love (const int groupIndex);
  const Status hate (const int groupIndex);

  BufMgr * getBufMgr ();

  const Status flushBufferPool ();
  const Status dumpHitRate ();

  // total number of frames a group is allocated on disk
  static int framesPerGroup;

  // just a helpful number to have around
  static int blocksPerFrame;

 // size of each buffer -- staticly compiled in for now
  //  static int frameSize;

  // number of frames in buffer pool
  static int frameCount;

  // number of groups
  static int groupCount;

  // how many sectors does the disk contain
  //  static int sectorCount;

 private:
  // /dev/sda....
  const char *const rawDiskDevice;
  Disk *disk;

  // the one true buffer manager
  BufMgr* bufMgr;

  // an array of groups
  Group **group;

#ifdef QUICK_CFS_LOOKUP
  // an array of their fileTables (for faster lookup)
  FileTable **fileTable;
#endif

  pthread_mutex_t quickMutex;
  QuickDigest *quickDigestListFront;
  const int quickDigestListLength;

  void genMD5 (const char *url, md5_byte_t *digest);
  //  void genMD5 (const char *url, char *digest);
  void printMD5 (md5_byte_t *digest);
 

  int filesWritten;
  int filesRead;
};

#endif
