/*
 * Copyright (C) 2001  Jonathan T. Ledlie
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "deleteQueue.h"

DeleteQueue::DeleteQueue (const int fCount) : frameCount (fCount) {

  cout << "ctor  deleteQueue" << endl;
  queue = new (RIDnode*) [frameCount];
  for (int i = 0; i < frameCount; i++) {
    queue[i] = NULL;
  }

}

DeleteQueue::~DeleteQueue () {
  //  cout << "deleting deleteQueue" << endl;
  delete [] queue;
}

Status DeleteQueue::enqueue (RID rid) {
  RIDnode *newNode = new RIDnode (rid);

  // queue for this page was empty
  if (queue [rid.pageNo] == NULL) {
    queue [rid.pageNo] = newNode;
    return OK;
  }

  RIDnode *current = queue [rid.pageNo];

  if (rid.slotNo > current->slotNo) {
    // put it at the front
    newNode->next = current;
    queue[rid.pageNo] = newNode;
    return OK;
  }

  while (current->next != NULL) {
    if (current->slotNo  > rid.slotNo && current->next->slotNo < rid.slotNo) {
      newNode->next = current->next;
      current->next = newNode;
      return OK;
    }
    current = current->next;
  }

  // put it at the end
  current->next = newNode;

  return OK;
}

RID DeleteQueue::dequeue (int pageNo) {
  RID rid;
  rid.pageNo = pageNo;

  if (queue[pageNo] == NULL) {
    rid.slotNo = -1;
    return rid;
  }
  rid.slotNo = queue[pageNo]->slotNo;
  RIDnode *front = queue[pageNo];
  queue[pageNo] = front->next;
  delete front;
  return rid;

}

//=============================================================================
ostream& operator << (ostream& os, DeleteQueue& queue) {
  for (int i = 0; i < queue.frameCount; i++) {
    if (queue.queue[i] != NULL) {
      os << i << " ";
      RIDnode *current = queue.queue [i];
      while (current->next != NULL) {
	os << "(" << current->pageNo << "," << current->slotNo << ") ";
	current = current->next;
      }
      os << "(" << current->pageNo << "," << current->slotNo << ") ";
      os << endl;
    }
  }
  return os;
}




RIDnode::RIDnode (RID rid) {
  this->pageNo = rid.pageNo;
  this->slotNo = rid.slotNo;
  this->next = NULL;
}

RIDnode::~RIDnode () {
}
