/*
 * Copyright (C) 2001  Jonathan T. Ledlie
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DISK_H
#define DISK_H

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <errno.h>
#include "globals.h"
#include "request.h"
#include "requestQueue.h"
#include "cfs.h"

class Disk {
  friend ostream& operator << (ostream& os, Disk& disk);
  friend RequestQueue;
 public:
  Disk (const char *fileName, Status& returnStatus);
  ~Disk ();
  void run ();
  RequestQueue* getRequestQueue () { return requestQueue; }
  static int diskFrameCount;

 private:
  RequestQueue* requestQueue;
  int fd;
  int head;
  //  const int numFrames;
  bool quitting;

  Status doWrite (Request *request);
  Status doRead (Request *request);

  // statistics
  int totalSeekDistance;       // incremented on each seek
  int maxRequestQueue;    // longest the queue has ever been


};

#endif
