/*
 * Copyright (C) 2001  Jonathan T. Ledlie
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <pthread.h>
#include "status.h"
#include "disk.h"
#include "file.h"

struct SampleFrame {
  char data;
};

void * runDisk (void * arg);

int main () {

  const int fileCount = 10;
  const int framesPerFile = 10;

  SampleFrame frame [fileCount][framesPerFile];

  for (int i = 0; i < fileCount; i++) {
    for (int j = 0; j < framesPerFile; j++) {
      frame[i][j].data = (char)(50 + (i+1)*(j +2));
      cout << frame[i][j].data << endl;
    }
  }

  Status status;

  pthread_t diskThread;

  int diskSectors = 4000;
  int sectorsPerGroup = diskSectors / fileCount;
  int framesPerGroup = sectorsPerGroup / (frameSize / BLOCK_SIZE);

  // 16 groups, 20 sectors each
  Disk *disk = new Disk ("/home/ledlie/proxy/cfs_cache/data", diskSectors, status);
  ASSERTSTATUS (status == OK);
  pthread_create (&diskThread, NULL, &runDisk, (void *) disk);

  //  pause ();

  int offsetBlock = 0;
  File *file[fileCount];

  for (int i = 0; i < fileCount; i++) {
    file[i] = new File (i, disk->getRequestQueue(), offsetBlock, framesPerGroup);
    offsetBlock += sectorsPerGroup;
    cout << "offset block" << offsetBlock << endl;
  }

  char *data = new char [frameSize];
  char *verifyData = new char [frameSize];

  cout << "Writing......" << endl;

  for (int i = 0; i < fileCount; i++) {
    for (int j = 0; j < framesPerFile; j++) {
      //cout << i << "\t";
      memset (data, frame[i][j].data, frameSize);
      int pageNo;
      status = file[i]->allocatePage (pageNo);
      ASSERTSTATUS (status == OK);

      ASSERT (pageNo == j);

      status = file[i]->writePage (pageNo, (Page*)data);
      ASSERTSTATUS (status == OK);
    }
  }
  cout << "Reading......" << endl;

  for (int i = 0; i < fileCount; i++) {
    for (int j = 0; j < framesPerFile; j++) {
      //    cout << i << "\t";
      memset (verifyData,frame[i][j].data , frameSize);

      status = file[i]->readPage (j, (Page*)data);
      ASSERTSTATUS (status == OK);

      int memStatus = memcmp (verifyData, data, frameSize);
      //      cout << "mem " << memStatus << endl;
      //      cout << "data\n" << data << endl;
      //      cout << "verify\n" << verifyData << endl;
      ASSERT (memStatus == 0);
    }
  }

  //  for (int i = 0; i < fileCount; i++) {
  for (int i = fileCount - 1; i >= 0; i--) {
    for (int j = framesPerFile - 1; j >= 0; j--) {
      //    cout << i << "\t";
      memset (verifyData,frame[i][j].data , frameSize);

      status = file[i]->readPage (j, (Page*)data);
      ASSERTSTATUS (status == OK);

      int memStatus = memcmp (verifyData, data, frameSize);
      //      cout << "mem " << memStatus << endl;
      //      cout << "data\n" << data << endl;
      //      cout << "verify\n" << verifyData << endl;
      ASSERT (memStatus == 0);
    }
  }


  delete [] data;

  delete disk;
  return 0;

}

void * runDisk (void * arg) {
  ((Disk*)arg)->run ();
}
