/*
 * Copyright (C) 2001  Jonathan T. Ledlie
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef FFS_H
#define FFS_H

#include <iostream.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>
#include "md5.h"
#include "status.h"

// #include "fs.h"

struct FileDesc {
  md5_byte_t digest [16];
  char dirA;
  char dirB;
  // bad jon!
  char fileName [120];
};

class FFS {
friend ostream& operator << (ostream& os, FFS& ffs);
 public:
 FFS (const char *const baseDirectory, Status& returnStatus); 
 ~FFS (); 
 Status lookup (const char *const url, int & groupNumber);
 Status create (const char *const url, const int group, const char *const data, const int size);
 Status read (const char *const url, const int group, char *&data, int &length);
 Status close (const char *const url);
 Status remove (const char *const url, int groupNumber);

 private:
 void genDesc (const char *url, FileDesc &desc);
 void genMD5 (const char *url, md5_byte_t *digest);
 void printMD5 (md5_byte_t *digest);
 const char *baseDirectory;
 int filesRead;
 int filesWritten;
};

#endif
