/*
 * Copyright (C) 2001  Jonathan T. Ledlie
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef FILE_H
#define FILE_H

#include <string>
#include <iostream>
#include <vector>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <errno.h>
#include "globals.h"
#include "status.h"
#include "page.h"
#include "request.h"
#include "requestQueue.h"

#include "status.h"
#include "page.h"

class DeleteQueue;
class Group;
class File;

class RIDnode {
  friend DeleteQueue;
 public:
  RIDnode (RID rid);
  ~RIDnode ();
  int pageNo;
  int slotNo;
  RIDnode *next;  
};

class DeleteQueue {
  friend ostream& operator << (ostream& os, DeleteQueue& queue);
  friend File;
 public:
  DeleteQueue (int frameCount);
  ~DeleteQueue ();
  Status enqueue (RID rid);
  Status dequeue (int pageNo, RID &rid);

 private:
  RIDnode **queue;
  const int frameCount;
  pthread_mutex_t mutex;
};

class File {
  friend Group;
  friend ostream& operator << (ostream& os, File &file);
 public:
  //  File (const string & fname, BufMgr *bufMgr);
  File (const int id, RequestQueue *queue, const int offset, int frameCount);
  ~File ();
  const Status writePage (const int pageNo, Page* pagePtr);
  const Status readPage (const int pageNo, Page* pagePtr);
  const Status allocatePage (int& pageNo);

  const int id;
 private:
  RequestQueue *queue;
  DeleteQueue *deleteQueue;
  const int offset;
  int numFrames;
  const int maxFrames;
  //  pthread_mutex_t mutex;

  vector<int> pagesInMemory;
};

#endif
