/*
 * Copyright (C) 2001  Jonathan T. Ledlie
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "fileTable.h"

int FileTable::hash (const md5_byte_t *const digest) {
      
   //   const int alpha = 23;
  int key;
  memcpy (&key, digest + 6, sizeof (int));
  //cout << key << endl;
  int index = abs (key % FILE_TABLE_NUM_CHAINS);

  //  int sum = 0;
//    int prev = 0;
//    for (int i = 0; i < DIGEST_LENGTH; i++) {
//      int tmp = digest[i];
//      sum += alpha * prev + tmp;
//      prev = tmp;
//    }
//    int index = sum % FILE_TABLE_NUM_CHAINS;

//   int key, key1, key2, key3, key4;

//   memcpy (&key1, digest, sizeof (int));
//   memcpy (&key2, digest + 4, sizeof (int));
//   memcpy (&key3, digest + 8, sizeof (int));
//   memcpy (&key4, digest + 12, sizeof (int));

//   key1 = key1 | key3;
//   key2 = key2 | key4;
//   key = key1 | key2;

//   //  cout << key << endl;
//   int index = abs (key % FILE_TABLE_NUM_CHAINS);

  return index;
}

FileTable::FileTable () {
  pthread_mutex_init (&mutex, NULL);
  recordCount = 0;
  for (int i = 0; i < FILE_TABLE_NUM_CHAINS; i++) {
    chain[i] = NULL;
  }
}

FileTable::~FileTable () {
  for (int i = 0; i < FILE_TABLE_NUM_CHAINS; i++) {
    FileDesc *current = chain[i];
    FileDesc *previous = NULL;
    while (current != NULL) {
      previous = current;
      current = current->next;
      delete previous;
    }
  }
}

//-----------------------------------------------------------------------------------
const Status FileTable::insert (const md5_byte_t *const digest, const RID rid) {

#ifdef DEBUG_FILETABLE
  cout << "top of FileTable::insert" << endl;
#endif

//Status FileTable::insert (const md5_byte_t *const digest, const RID rid) {
  //  int key;

  //  memcpy (&key, digest, sizeof (int));
  //  int index = abs (key % FILE_TABLE_NUM_CHAINS);

  int index = FileTable::hash (digest);

  FileDesc *desc = new FileDesc (digest, rid);
  if (desc == NULL) return OUT_OF_MEMORY;

  pthread_mutex_lock (&mutex);  

  if (chain[index] == NULL) {
    chain[index] = desc;
  } else {
    desc->next = chain[index];
    chain[index] = desc;
  }

  recordCount++;
  pthread_mutex_unlock (&mutex);  
  return OK;
}

#ifdef QUICK_CFS_LOOKUP
const Status FileTable::lookup (const md5_byte_t *const digest, const int chainIndex) {

  pthread_mutex_lock (&mutex);  
  FileDesc *current = chain[chainIndex];

  while (current != NULL) {
    if (memcmp (current->digest, digest, 16) == 0) {
      pthread_mutex_unlock (&mutex);  
      return OK;
    } else {
      current = current->next;
    }
  }
  pthread_mutex_unlock (&mutex);  
  return FILE_NOT_FOUND;
}
#endif

//-----------------------------------------------------------------------------------
const Status FileTable::lookup (const md5_byte_t *const digest, RID& outRid) {

#ifdef DEBUG_FILETABLE
  cout << "top of FileTable::lookup" << endl;
#endif

  int chainIndex = FileTable::hash (digest);

#ifdef USE_FAST_MD5
  int probeKey = 0;
  memcpy (&probeKey, digest + 12, sizeof(int));
#endif

  pthread_mutex_lock (&mutex);  
  FileDesc *current = chain[chainIndex];

#ifdef DEBUG_FILETABLE
  cout << "entering loop in FileTable::lookup" << endl;
#endif

  while (current != NULL) {
#ifdef USE_FAST_MD5
    if ((probeKey == current->key) && (memcmp (current->digest, digest, 16) == 0)) {
#else
    if (memcmp (current->digest, digest, 16) == 0) {
#endif
      outRid = current->rid;
      pthread_mutex_unlock (&mutex);  
      return OK;
    } else {
      current = current->next;
    }
  }
  pthread_mutex_unlock (&mutex);  
  return FILE_NOT_FOUND;
}

//-----------------------------------------------------------------------------------
const Status FileTable::remove (const md5_byte_t *const digest) {
  //Status FileTable::remove (const md5_byte_t * const digest)  { //FileDesc &deleteMe

#ifdef DEBUG_FILETABLE
  cout << "top of FileTable::remove" << endl;
#endif

  //  int key;
  //  memcpy (&key, digest, sizeof (int));
  //  int index = abs (key % FILE_TABLE_NUM_CHAINS);
  int index = FileTable::hash (digest);


#ifdef USE_FAST_MD5
  int probeKey = 0;
  memcpy (&probeKey, digest + 12, sizeof(int));
#endif

  pthread_mutex_lock (&mutex);  
  FileDesc *current =  chain[index];
  FileDesc *previous = NULL;

  while (current != NULL) {
#ifdef USE_FAST_MD5
    if ((probeKey == current->key) && (memcmp (current->digest, digest, 16) == 0)) {
#else
      if (memcmp(current->digest,digest, DIGEST_LENGTH) == 0) {
#endif
	//      deleteMe.length = current->length;
	//deleteMe.start = current->start;
      if (chain[index] == current)
	chain[index] = current->next;
      else
	previous->next = current->next;
      delete current;
      recordCount--;
      //      found = true;
      //      cout << "deleted record" << endl;
      pthread_mutex_unlock (&mutex);  
      return OK;
    } else {
      previous = current;
      current = current->next;
    }
  }
  pthread_mutex_unlock (&mutex);  

  cout << "did not delete record" << endl;

  //  if (found)
  //    return OK;
  return FILE_NOT_FOUND;
}


ostream& operator <<( ostream & os, FileTable& ft)  {
  int total = 0;
  int max = 0;
  int numZilch = 0;
  //  int  totalFileSize = 0;
  for (int i = 0; i < FILE_TABLE_NUM_CHAINS; i++) {
    FileDesc *current = ft.chain[i];
    int found = 0;
    while (current != NULL) {
      //      totalFileSize += current->length;
      current = current->next;
      found++;
    }
    total += found;
    if (found == 0) numZilch++;
    if (max < found) max = found;
  }
  os << "file count: " << ft.recordCount << " longest chain: " << max << " avg chain: " << total/FILE_TABLE_NUM_CHAINS << " zero length/total: " << numZilch << "/" << FILE_TABLE_NUM_CHAINS;

  // << " avgFileSize: ";
  // if (ft.recordCount == 0) 
//     os << "0";
//   else 
//     os << totalFileSize / ft.recordCount;
  return os;
}

FileDesc::FileDesc (const md5_byte_t *pDigest, const RID inRid) : rid (inRid) {
  memcpy (&digest, pDigest, 16);

  //  key = -1;

  // what did this do???
#ifdef USE_FAST_MD5
  memcpy (&(this->key), pDigest + 12, sizeof(int));
#endif

  // this->rid = inRid
    //  this->length = length;
  next = NULL;
}

// FileDesc::FileDesc () {
//   memset (digest, 0, 16);
//   key = -1;
//   rid.pageNo = -1;
//   rid.slotNo = -1;
//   next = NULL;
// }

FileDesc::~FileDesc () {
}
