/*
 * Copyright (C) 2001  Jonathan T. Ledlie
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef FILE_TABLE_H
#define FILE_TABLE_H

#include "md5.h"
#include "page.h"
#include <pthread.h>
#define _REENTRANT
#include "globals.h"

//#define USE_FAST_MD5

class FileDesc {
  friend ostream& operator << (ostream& os, FileDesc& fileDesc);

 public:
  FileDesc (const md5_byte_t *const digest, const RID rid);
  ~FileDesc ();
  md5_byte_t digest[DIGEST_LENGTH];
  RID rid;
  FileDesc *next;
#ifdef USE_FAST_MD5
  int key;
#endif
};

class FileTable {
  friend ostream& operator << (ostream& os, FileTable& fileTable);

 public:
  FileTable ();
  ~FileTable ();
  const Status lookup (const md5_byte_t *const digest, RID& outRid);
  const Status insert (const md5_byte_t *const digest, const RID Rid);
  const Status remove (const md5_byte_t *const digest);

  static int hash (const md5_byte_t *const digest);

#ifdef QUICK_CFS_LOOKUP
  const Status lookup (const md5_byte_t *const digest, const int chainIndex);
#endif

 private:  
  FileDesc *chain [FILE_TABLE_NUM_CHAINS];
  pthread_mutex_t mutex;
  int recordCount;

};

#endif
