/*
 * Copyright (C) 2001  Jonathan T. Ledlie
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

// #include <stdlib.h>
// #include <string.h>
// #include <errno.h>
// #include <unistd.h>
// #include <iostream.h>
// #include <sys/types.h>
// #include <sys/socket.h>
// #include <netinet/in.h>
// #include <arpa/inet.h>
// #include <iostream.h>
#include "proxyThread.h"

// clear out data structures for next request
 void ProxyThread::clear () {
  //  request.url = NULL;
  memset (request.url, 0, MAX_URL);
  memset (request.data, 0, MAX_LINE);
  request.length = -1;
  memset (&request.serverAddress, 0, sizeof (request.serverAddress));
  request.port = -1;
  request.refresh = false;
  request.cacheable = true;
  request.socketFD = -1;
  for (int i = 0; i < MAX_HEADER_COUNT; i++) {
    memset (request.headers[i], 0, MAX_HEADER_WIDTH);
  }

  //  memset (&response.data, 0, sizeof (response.data));
  for (int i = 0; i < MAX_HEADER_COUNT; i++) {
    memset (response.headers[i], 0, MAX_HEADER_WIDTH);
  }
  response.headerLength = 0;
  memset (response.headerData, 0, MAX_LINE);
  if (response.data != NULL) {
    cerr << "leaking memory" << endl;
    response.data = NULL;
  }
  response.length = -1;
  response.socketFD = -1;
}

/* Read, at most maxlen bytes of data from the socket fd */
 ssize_t ProxyThread::readn(int fd, void *vptr, ssize_t maxlen)   {
  ssize_t nbytes = maxlen;
  char *ptr = (char *) vptr;
  while (nbytes) {
    ssize_t n = ::read(fd,ptr,nbytes);
    if (n < 0) {
      if (errno == EINTR) {
	n = 0;
	cout << "got eintr" << endl;
      }
      else
	return n;
    } else if (n == 0) {
      break;
    }
    ptr += n;
    nbytes -= n;
    cout << "still reading" << endl;
  }
  return maxlen-nbytes;
}

/* Write n bytes of data to the socket fd */
 ssize_t ProxyThread::writen (int fd, void *vptr, ssize_t n)   {
  //cout << "top of writen" << endl;
  size_t  nbytes = n;
  char *ptr = (char *) vptr;
  while (nbytes) {
    ssize_t nwritten = ::write(fd,ptr,nbytes);
    //cout << "nwritten = " << nwritten << endl;
    if (nwritten <= 0) {
      if (errno == EINTR) {
	//cout << "Got EINTR" << endl;
	nwritten = 0;
      }  else if (errno ==  ECONNRESET) {
	//cout << "Got ECONNRESET" << endl;
	return (n - nbytes);
      } else {
	//	cout << errno2str () << endl;
	//cout << errno << endl;
	//perror ("Error in writen:");
	return nwritten;
      }
    }
    nbytes -= nwritten;
    ptr += nwritten;
  }
  
  //cout << "wrote " << n - nbytes << " bytes" << endl;

  return (n - nbytes);
}


 long ProxyThread::str2time (char *timeString) {
  long result = 0;
  long days = 0;
  // Fri, 15 Apr 1994 00:00:00
  // 0123456789012345678901234
  char yearString[5];
  yearString[0] = *(timeString + 12);
  yearString[1] = *(timeString + 13);
  yearString[2] = *(timeString + 14);
  yearString[3] = *(timeString + 15);
  yearString [4] = '\0';  
  //  cout << "yearString: " << yearString << endl;
  int year = atoi (yearString) - 1970;
  days = (year * 365) + (year / 4);
  // forget about leap years for now
  //  cout << "days: " << days << endl;

  char monthString[4];
  monthString[0] = *(timeString + 8);
  monthString[1] = *(timeString + 9);
  monthString[2] = *(timeString + 10);
  monthString [3] = '\0';  
  //  cout << "mon " << monthString << endl;

  if (monthString [0] == 'J') {
    if (monthString [2] == 'u') {
      if  (monthString [2] == 'n') {
	// jun
	days += 151;
      } else {
	// july
	days += 181;
      }
    }
  } else if (monthString [0] == 'F') {
    days += 31;
  } else if (monthString [0] == 'M') {
    if (monthString [2] == 'y') {
      // may
      days += 120;
    } else {
      // march
      days += 59;
    }
  } else if (monthString [0] == 'A') {
    if (monthString [1] == 'u') {
      days += 212;
    } else {
      // april
      days += 90;
    }
  } else if (monthString [0] == 'S') {
    days += 243;
  } else if (monthString [0] == 'O') {
    days += 273;
  } else if (monthString [0] == 'N') {
    days += 304;
  } else if (monthString [0] == 'D') {
    days += 334;
  } else {
    days += 0;
  }

  //  cout << "days: " << days << endl;

  char dayOfMonth [3];
  dayOfMonth[0] = *(timeString + 5);
  dayOfMonth[1] = *(timeString + 6);
  dayOfMonth [2] = '\0';
  days += atoi (dayOfMonth);

  result = days * 24 * 60 * 60;

  char hours [3];
  hours [0] = *(timeString + 17);
  hours [1] = *(timeString + 18);
  hours [2] = '\0';
  result += (atoi (hours) * 60 * 60);

  char minutes [3];
  minutes [0] = *(timeString + 20);
  minutes [1] = *(timeString + 21);
  minutes [2] = '\0';
  result += (atoi (minutes) * 60);

  char seconds [3];
  seconds [0] = *(timeString + 23);
  seconds [1] = *(timeString + 24);
  seconds [2] = '\0';
  //  printf ("seconds: %c%c atoi %d\n", seconds[0], seconds[1], atoi(seconds));
  result += atoi (seconds);

  //  cout << "y:" << year << "h:" << hours << " m:" << minutes << " s:" << seconds << endl;  
  //  cout << "days: " << days << endl;
  //  cout << "result: " << result << endl;
  return result;

}


 Status ProxyThread::openServerSocket () {

#ifdef DEBUG_FUNCTION_CALLS
  cout << "open server socket" << endl;
#endif

  char hostname [MAX_HEADER_WIDTH];
  memset (hostname, 0, MAX_HEADER_WIDTH);
  char *hostStart = strstr (request.headers[0], "//");
  hostStart += 2;

  /*   if (hostStart == NULL) { */
  /*     return BAD_CLIENT_HEADER; */
  /*   } */

  //  cout << "start: " << hostStart << endl;
  char *hostEnd = strchr (hostStart, '/');

  char *colon;
  if ( (colon = strchr (hostStart, ':')) != NULL) {
    //    cout << "found colon";
    char portString[MAX_PORT_CHARS];
    memcpy (hostname, hostStart,  colon - hostStart);
    memcpy (portString, colon + 1, hostEnd - colon - 1);
    //    cout << "port: " << portString << "E" << endl;
    request.port = atoi (portString);
  } else {
    memcpy (hostname, hostStart, hostEnd - hostStart);
    request.port = 80;
  }

  //  cout << "hostname: " << hostname  << "port: " << request.port << endl;

  // struct hostent stackSpace;
  struct hostent *serverEntity;
  //  struct hostent serverEntity;
  h_errno = -1;
  //  serverEntity = NULL;
  serverEntity = gethostbyname (hostname);
  //  while (h_errno < 0) {
  while (serverEntity == NULL) {
    serverEntity = gethostbyname (hostname);
    cout << hstrerror (h_errno) << endl;
    sleep (1);
  }
  
  if (serverEntity == NULL) {
    cout << "serverEntity is null" << endl;
    // proxyExit (0);
    return NAME_LOOKUP_ERROR;
  }

  struct sockaddr_in serverAddress;
  memset (&serverAddress, 0, sizeof (serverAddress));
  serverAddress.sin_family = AF_INET;
 
  struct in_addr **pptr;
  pptr = (struct in_addr **) serverEntity->h_addr_list;
  memcpy (&serverAddress.sin_addr, *pptr, sizeof (struct in_addr));

  //  char str [INET_ADDRSTRLEN];
  //  for ( ; *pptr != NULL; pptr++) {
  //    printf ("Address: %s\n", inet_ntop (serverEntity->h_addrtype, *pptr, str, sizeof (str)));
  //  }

  //  ::printIP (pptr);

  serverAddress.sin_port = htons (request.port);

  if ( (response.socketFD = socket (AF_INET, SOCK_STREAM, 0)) < 0) {
    //    cout << "upstream proxy: can't open stream socket" << endl;
    perror ("Upstream proxy: can't open stream socket");
    return SERVER_CONNECT_ERROR;
  }

  if (connect (response.socketFD, (struct sockaddr *) &serverAddress, sizeof (serverAddress)) < 0) {
    //    cout << "upstream proxy: can't connect to server" << endl;
    perror ("Upstream proxy: can't connect to server");
    return SERVER_CONNECT_ERROR;
  }

#ifdef DEBUG_FUNCTION_CALLS
  cout << "end of open server socket" << endl;
#endif

  return OK;
}
