/*
 * Copyright (C) 2001  Jonathan T. Ledlie
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "microShared.h"
#include "group.h"
#include "cfs.h"

SampleFile *file;

bool zipfAccess;
int zipf;

int **createList;
int **readList;
int **deleteList;
int operationCount;


int fileCount = 0;
char baseOutputDir[] = "/mnt/scsi_c/log/cfs/";
bool logCreates = false;
bool logReads = false;
bool logDeletes = false;

int groupCount = 0;
int bufferFrameCount = 0;

// extern long timeInRead;
// extern long timeInCreate;
// extern long timeInDelete;

CFS *cfs;

int main(int argc, char **argv) {

  Status status;
  
  setup (argc, argv);

  cfs = new CFS ("/dev/raw/raw4", groupCount, bufferFrameCount, status);
  //  cfs = new CFS ("/u/l/e/ledlie/private/thesis/cfs_cache/data", groupCount, bufferFrameCount, status);
  //  cfs = new CFS ("/u/l/e/ledlie/private/thesis/cfs_cache/data", groupCount, bufferFrameCount, status);
  if (status != OK) cout << status << endl;

  startThreads ();

  VERBOSE (cfs);
  delete cfs;

}

void * createLoop (void * arg) {
  Status status;
  int id = (int) arg;

  LOG (FILE *log);
  if (logCreates)
    LOG (log = openLog (id, 'c'));

  VERBOSE ("Starting thread"  << id);

  //  sleep (2);

  timeval threadStartTime;
  if (logCreates)
    gettimeofday (&threadStartTime, NULL);

  VERBOSE ("*** 1 Creates...." << id);

// #ifdef  VERBOSE
//   if (id == 0)
//     cout << cfs << endl;
// #endif

  char *dataArray = new char[FILE_SIZE_RANGE];
  for (int i = 0; i < fileCount; i++) {
#ifdef DEBUG_FREQUENCY
    if (i % DEBUG_FREQUENCY == 0) {
      cout << id << " created " << i << endl;
    }
#endif
    //   cout << "id: " << id << " i: " << i << endl;
    //  cout << createList[id][i] << endl;
    //    cout << "in micro: " << file[createList[id][i]].name << endl;
    if (createList[id][i] != -1) {
      memset (dataArray, file[createList[id][i]].data, file[createList[id][i]].length);
      status = cfs->create (file[createList[id][i]].name, file[createList[id][i]].groupNumber, dataArray, file[createList[id][i]].length);
      ASSERTSTATUS (status == OK);
      LOG (if (logCreates) printTime (log, threadStartTime));
    }
  }

// #ifdef  VERBOSE
//   if (id == 0)
//     cout << cfs << endl;
// #endif

  delete [] dataArray;
  if (logCreates)
    LOG (fclose (log));
  pthread_exit ((void *) id);
  return NULL;
}


void * readLoop (void * arg) {
  Status status;
  int id = (int) arg;

  LOG (FILE *log);
  if (logReads)
    LOG (log = openLog (id, 'r'));



  if (id == 0 && zipfAccess) {
    for (int i = 0; i < groupCount; i++) {
      if (i < (zipf * groupCount) / 8 ) {
	cfs->love (i);
	cout << "Loving group " << i << endl;
      } else {
	cfs->hate (i);
	cout << "Hating group " << i << endl;
      }
    }
  }




  char *dataArray = new char[FILE_SIZE_RANGE];

  timeval threadStartTime;
  if (logReads)
    gettimeofday (&threadStartTime, NULL);

  VERBOSE ("*** 2 Reads...." << id);

  for (int i = 0; i < fileCount; i++) {
#ifdef DEBUG_FREQUENCY
    if (i % DEBUG_FREQUENCY == 0) {
      cout << id << " read " << i << endl;
    }
#endif
    if (readList[id][i] != -1) {
      //      cout << "r " << file[i].name << endl;
      memset (dataArray, file[readList[id][i]].data, file[readList[id][i]].length);
      status = OK;
      int groupNumber;
      status = cfs->lookup (file[readList[id][i]].name, groupNumber);

      //if (id == 0) cout << "lookup: " << groupNumber << " lookup2 " << groupNumber2 << endl;

      ASSERTSTATUS (status == OK);
      char *readPtr;
      int readLength;
      status = cfs->read (file[readList[id][i]].name, groupNumber, readPtr, readLength);
      ASSERTSTATUS ((status == OK) && (file[readList[id][i]].length == readLength) && (memcmp (dataArray, readPtr,readLength) == 0));
    
      status = cfs->release (file[readList[id][i]].name, groupNumber);
      ASSERTSTATUS (status == OK);
      LOG (if (logReads) printTime (log, threadStartTime));
    }
  }

  delete [] dataArray;
  if (logReads)
    LOG (fclose (log));

  pthread_exit (0);
  return NULL;
}

void * deleteLoop (void * arg) {
  Status status;
  int id = (int) arg;

  LOG (FILE *log);
  if (logDeletes)
    LOG (log = openLog (id, 'd'));

  if (id == 0 && zipfAccess) {
    for (int i = 0; i < groupCount; i++) {
      cfs->love (i);
      cout << "Loving group " << i << endl;
    }
  }



  timeval threadStartTime;
  if (logDeletes)
    gettimeofday (&threadStartTime, NULL);

  VERBOSE ("*** 3 Deletes...." << id);

  for (int i = 0; i < fileCount; i++) {
#ifdef DEBUG_FREQUENCY
    if (i % DEBUG_FREQUENCY == 0) {
      cout << id << " deleted " << i << endl;
    }
#endif

    if (deleteList[id][i] != -1) {

      int groupNumber;
      //      if (id == 0) cout << "d " << i << " " << file[i].name << endl;

      //  cout << id << " deleting " << i << "th: " << deleteList[id][i] << endl;

      status = OK;
      status = cfs->lookup (file[deleteList[id][i]].name, groupNumber);

      

      // if (id == 0) cout << "lookup: " << groupNumber << " lookup2 " << groupNumber2 << endl;

   
      ASSERTSTATUS (groupNumber == file[deleteList[id][i]].groupNumber);
      status = cfs->remove (file[deleteList[id][i]].name, groupNumber);
      ASSERTSTATUS (status == OK);
      LOG (if (logDeletes) printTime (log, threadStartTime));
    }
  }

  if (logDeletes)
    LOG (fclose (log));

  pthread_exit (0);
  return NULL;
}


void flushBufferPool () {
  cfs->flushBufferPool ();
}

void dumpHitRate () {
  cfs->dumpHitRate ();
}
