/*
 * Copyright (C) 2001  Jonathan T. Ledlie
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include <unistd.h>
#include "microShared.h"

// extern SampleFile *file;
// extern int fileCount;
// extern long timeInRead;
// extern long timeInCreate;
// extern long timeInDelete;

int threadCount;

void genRandomArray(int *nums, int len);


const char usage[] = "Usage: micro [-n number of web objects] [-g group count] [-m buffer pool size in megabytes] [-t c|r|d] [-s sequential| -r random | -z zipf] [-x random seed]";



void setup (int argc, char **argv) {

  bool sequentialAccess = true;
  bool randomAccess = false;
  zipfAccess = false;
  bool randomSeed = false;



  int bufferPoolSize = 0;

  //GetOpt getopt (argc, argv, "n:g:f:m:t:srx:");
  //int option_char;

  int c;
  
  // Invokes member function `int operator ()(void);'
 while ((c = getopt (argc, argv, "n:g:f:m:t:srz:x:")) != -1)
    switch (c) {  
    case 'n':
      fileCount = atoi (optarg);
      break;
    case 'g':
      groupCount = atoi (optarg);
      threadCount = groupCount;
      break;
    case 'm':
      bufferPoolSize = atoi (optarg) * 1024 * 1024;
      break;
    case 't':
      if (*optarg == 'c') {
	logCreates = true;
      } else if (*optarg == 'r') {
	logReads = true;
      } else if (*optarg == 'd') {
	logDeletes = true;
      } else if (*optarg == 'a') {
	logCreates = true;
	logReads = true;
	logDeletes = true;
      }
      break;
    case 'z':
      zipf = atoi (optarg);
      zipfAccess = true;
      sequentialAccess = false;
      randomAccess = false;
      break;
    case 's':
      sequentialAccess = true;
      randomAccess = false;
      zipfAccess = false;
      break;
    case 'r':
      randomAccess = true;
      sequentialAccess = false;
      zipfAccess = false;
      break;
    case 'x':
      srand (atoi(optarg));
      randomSeed = true;;
      break;
    case '?':
      cerr << usage << endl;
      exit (-1);
    }

 if (fileCount == 0 || groupCount == 0 || frameSize == 0 || bufferPoolSize == 0) {
   cerr << usage << endl;
   exit (-1);
 }

 if (zipfAccess && !(zipf == 1 || zipf == 2 || zipf == 3 || zipf == 4)) {
   cerr << usage << endl;
   exit (-1);
 }

 if (zipfAccess && (fileCount % groupCount != 0)) {
   cerr << "Group count must evenly divide file count." << endl;
   cerr << usage << endl;
   exit (-1);
 }


 bufferFrameCount = bufferPoolSize / frameSize; 

  if (!randomSeed)
    srand (getpid());

  file = new SampleFile [fileCount];

  int totalSize = 0;
  for (int i = 0; i < fileCount; i++) {
    if (randomAccess) 
      file[i].groupNumber = rand() % groupCount;
    else 
      //      file[i].groupNumber = i % groupCount;
      file[i].groupNumber = i / (fileCount / groupCount);
    file[i].dirA = (char)(97 + (i / (fileCount / 26 + 1)));
    file[i].dirB = (char)(97 + rand() % 26);

    file[i].length = 1 + rand() % FILE_SIZE_RANGE;
    totalSize += file[i].length;
    file[i].data = (char)(97 + rand() % 26);
    memset (file[i].name, 0, MAX_FILENAME);
    sprintf (file[i].name, "dummy.%d.%d", file[i].groupNumber, i);
  }
  cout << totalSize << " bytes needed to hold files." << endl;


  operationCount = fileCount / threadCount;
  createList = new int* [threadCount];
  readList = new int* [threadCount];
  deleteList = new int* [threadCount];
  for (int i = 0; i < threadCount; i++) {
    createList[i] = new int [fileCount];
    readList[i] = new int [fileCount];
    deleteList[i] = new int [fileCount];
  }

  for (int i = 0; i < threadCount; i++) {
    for (int j = 0; j < fileCount; j++) {
      createList[i][j] = -1;
      readList[i][j] = -1;
      deleteList[i][j] = -1;
    }
  }

  if (sequentialAccess) {
    cout << "Sequential Access" << endl;
    for (int i = 0; i < fileCount; i++) {
      createList[i % threadCount][i / threadCount] = i;
      //      cout << "list: " << i << ", " << i % threadCount << ", " << i / threadCount << endl;
    }

    for (int i = 0; i < threadCount; i++) {
      for (int j = 0; j < fileCount; j++) {
	//	cout << "(" << i << "," << j << ")=" << createList[i][j] << " ";
	readList[i][j] = createList[i][j];
	deleteList[i][j] = createList[i][j];

      }
      //      cout << endl;
    }


  } else {
    if (randomAccess) {
      cout << "Random Access" << endl;
    } else if (zipfAccess) {
      cout << "Zipf Access" << endl;
    } else {
      ASSERT (false);
    }

    int *createOperations = new int [threadCount];
    int *readOperations = new int [threadCount];
    int *deleteOperations = new int [threadCount];

    for (int i = 0; i < threadCount; i++) {
      createOperations[i] = 0;
      readOperations[i] = 0;
      deleteOperations[i] = 0;
    }


    for (int i = 0; i < fileCount; i++) {
      int whichThread = rand() % threadCount;
      createList[whichThread][createOperations[whichThread]++] = i;
    }

    if (randomAccess) {

      for (int i = 0; i < threadCount; i++) {    
	for (int j = 0; j < operationCount; j++) {    
	  readList[i][j] = rand() % fileCount;
	}
      }
    } else {

      int zipfPoint = int ((zipf / 8.0) * fileCount);
      int gtZipfPoint = fileCount - zipfPoint;
      cout << "zipfPoint = " << zipfPoint << ", remainer = " <<  gtZipfPoint << endl;

      for (int i = 0; i < threadCount; i++) {    
	for (int j = 0; j < operationCount; j++) {
	  if (rand() % 8 >= zipf) {
	    // pick file from first groups

	    int inZipf = rand() % zipfPoint;
	    //	    cout << "in: " << inZipf << endl;
	    readList[i][j] = inZipf;
	  } else {
	    // pick one from infrequently used groups
	    int outZipf = zipfPoint + (rand() % gtZipfPoint);
	    //    cout << "out: " << outZipf << endl;	    
	    readList[i][j] = outZipf;

	  }
	}
      }
    }



    for (int i = 0; i < fileCount; i++) {
      int whichThread = rand() % threadCount;
      deleteList[whichThread][deleteOperations[whichThread]++] = i;
    }


    for (int i = 0; i < threadCount; i++) {    
      genRandomArray (createList[i], fileCount);
      genRandomArray (deleteList[i], fileCount);
    }



    //     for (int i = 0; i < threadCount; i++) {
//       cout << i << " is creating... " << endl;
//       for (int j = 0; j < fileCount; j++) {
// 	cout << createList[i][j] << " ";
//       }
//       cout << endl;

//       cout << i << " is reading... " << endl;
//       for (int j = 0; j < fileCount; j++) {
// 	cout << readList[i][j] << " ";
//       }
//       cout << endl;

//       cout << i << " is deleting... " << endl;
//       for (int j = 0; j < fileCount; j++) {
// 	cout << deleteList[i][j] << " ";
//       }
//       cout << endl;

    //     }
//     exit (0);


  }

  //exit (0);

#ifdef SHOWFILES
  cout << "i\tlen\tdata\tgroup\tA\tB\tname" << endl;
  for (int i = 0; i < fileCount; i++) {
    cout << i << "\t" << file[i].length << "\t" << file[i].data << "\t";
    cout << file[i].groupNumber << "\t" << file[i].dirA << "\t" << file[i].dirB << "\t";
    cout << file[i].name << endl;

//     cout << i << "\t" << createFile[i].threadID << "\t" << createFile[i].length << "\t" << createFile[i].data << "\t";
//     cout << createFile[i].name << endl;

//     cout << i << "\t" << readFile[i].threadID << "\t" << readFile[i].length << "\t" << readFile[i].data << "\t";
//     cout << readFile[i].name << endl;

//     cout << i << "\t" << deleteFile[i].threadID << "\t" << deleteFile[i].length << "\t" << deleteFile[i].data << "\t";
//     cout << deleteFile[i].name << endl;
//   }



  }
  exit (0);

#endif

}

void startThreads () {

  double createTime = 0;
  double readTime = 0;
  double deleteTime = 0;

  VERBOSE ("Starting testing");
  timeval startTime;
  gettimeofday (&startTime, NULL);

#ifdef NO_THREADS
  threadLoop (0);
#else

#ifdef STATS
  pthread_t stat_t;
  pthread_create (&stat_t, NULL, &statsLoop, NULL);
#endif

  pthread_t tid[threadCount];

  for (int i = 0; i < threadCount; i++) {
    pthread_create (&tid[i], NULL, &createLoop, (void *) i);
  }

  for (int i = 0; i < threadCount; i++) {
    pthread_join(tid[i], NULL);
    VERBOSE ("Thread " << i << " has finished")
  }
  

  createTime = printTime (stdout, startTime);

  if (logReads || logDeletes) {

    flushBufferPool ();

  gettimeofday (&startTime, NULL);

  for (int i = 0; i < threadCount; i++) {
    pthread_create (&tid[i], NULL, &readLoop, (void *) i);
  }

  for (int i = 0; i < threadCount; i++) {
    pthread_join(tid[i], NULL);
    VERBOSE ("Thread " << i << " has finished")
  }

  readTime = printTime (stdout, startTime);

#ifdef LOG_HIT_RATE
  dumpHitRate();
#endif
  }

  if (logDeletes) {
  gettimeofday (&startTime, NULL);

  for (int i = 0; i < threadCount; i++) {
    pthread_create (&tid[i], NULL, &deleteLoop, (void *) i);
  }

  for (int i = 0; i < threadCount; i++) {
    pthread_join(tid[i], NULL);
    VERBOSE ("Thread " << i << " has finished")
  }

  deleteTime = printTime (stdout, startTime);
  }

#endif

  VERBOSE ("All done.");

  cout << "Total time: ";
  printTime (stdout, startTime);

  createTime /= 1000;
  readTime /= 1000;
  deleteTime /= 1000;


  cout << "Create:\t" << createTime << endl;
  cout << "Read:\t" << readTime << endl;
  cout << "Delete:\t" << deleteTime << endl;
  cout << "Total:\t" << (createTime + readTime + deleteTime) << endl;

  delete [] file;
}


ostream& operator << (ostream& os, SampleFile& sampleFile) {
  os << sampleFile.name << "\t(len " << sampleFile.length << ", data " << sampleFile.data << ", group " << sampleFile.groupNumber << ")";
  return os;
}


double printTime (FILE* fd, const timeval startTime) {
  timeval currTime;
  gettimeofday (&currTime, NULL);

  int seconds = currTime.tv_sec - startTime.tv_sec;
  double current = (seconds * 1000000) + currTime.tv_usec - startTime.tv_usec;

  current = current / 1000;

  //  cout << currTime.tv_sec << "\t" << currTime.tv_usec << "\t" << current << endl;

  fprintf (fd, "%10.0f\n", current);

  return current;

}

FILE* openLog (int id, char op) {
  //  int fd;
  FILE* fd;

  int fileNameLength = strlen(baseOutputDir) + 5;
  char fileName[fileNameLength];
  //  memset (&fileName, 0, fileNameLength);
  
  sprintf (fileName, "%slog.%c.%d", baseOutputDir, op, id);
 
  cout << fileName << endl;

  fd = fopen (fileName, "w");
  if (fd == NULL) {
    perror ("Failure opening log file");
    exit (0);
  }

  //  exit (0);
  return fd;
}


SampleFile::SampleFile () { }

SampleFile::SampleFile (SampleFile &s) {
  this->dirA = s.dirA;
  this->dirB = s.dirB;
  this->data = s.data;
  this->length = s.length;
  this->groupNumber = s.groupNumber;
  //  this->threadID = s.threadID;
  memcpy (this->name, s.name, MAX_FILENAME);
}

void genRandomArray(int *nums, int len)
{
    for (int pass = 0; pass < NUMRANDOMIZEPASSES; pass++)
    {
      for (int i = 0; i < len; i++)  {
	// swap curr entry to new pos
	int newPos = rand() % len;
	
	int tmp = nums[newPos];

	nums[newPos] = nums[i];
	
	nums[i] = tmp; 
      }
    }

} // end genRandomIntArray
