/*
 * Copyright (C) 2001  Jonathan T. Ledlie
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef PAGE_H
#define PAGE_H

#include "status.h"
#include "globals.h"

struct RID{
    int  pageNo;
    int  slotNo;
  RID& operator=(const RID &rid) { 
    pageNo = rid.pageNo; 
    slotNo = rid.slotNo;
    return *this;
  }

  friend ostream& operator << (ostream& os, RID& rid);
};


struct Record
{
  void* data;
  int length;
};

// slot structure
struct slot_t {
  //        short	offset;  
  //           short	length;  // equals -1 if slot is not in use

  int	offset;  
  int	length;  // equals -1 if slot is not in use


};

const unsigned DPFIXED= sizeof(slot_t)+4*sizeof(int)+2*sizeof(int);

class Page {
 public:
  // for testing only!!
  //  Page ();
  //  ~Page ();
private:
  char 	data[frameSize - DPFIXED]; 
  //  char 	*data; 

    // really: these guys location is going to have to be set somehow

    slot_t 	slot[1]; // first element of slot array - grows backwards!
    int	slotCnt; // number of slots in use;
    int	freePtr; // offset of first free byte in data[]
    int	freeSpace; // number of bytes free in data[]
    int	recCnt;	// for alignment purposes
    int		nextPage; // forwards pointer
    int		curPage;  // page number of current pointer

public:
    void init(const int pageNo); // initialize a new page
    //    void setDataPtr (char *data) { this->data = data; }
    void dumpPage() const;       // dump contents of a page

    const int getNextPage() const; // returns value of nextPage
    void setNextPage(const int pageNo); // sets value of nextPage to pageNo
    const int getFreeSpace() const; // returns amount of free space
    //    const short getFreeSpace() const; // returns amount of free space

    int getRecCnt ();

    // inserts a new record (rec) into the page, returns RID of record 
    const Status insertRecord(const Record & rec, RID& rid);

    // delete the record with the specified rid
    const Status deleteRecord(const RID & rid);

    // returns RID of first record on page
    // returns  NORECORDS if page contains no records.  Otherwise, returns OK
    const Status firstRecord(RID& firstRid) const;

    // returns RID of next record on the page 
    // returns ENDOFPAGE if no more records exist on the page
    const Status nextRecord (const RID & curRid, RID& nextRid) const;

    // returns reference to record with RID rid
    const Status getRecord(const RID & rid, Record & rec);
};

#endif
