/*
 * Copyright (C) 2001  Jonathan T. Ledlie
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "proxy.h"

Proxy* Proxy::globalProxy;

void proxyExit (int signo) {
  Proxy *proxy = Proxy::globalProxy;

  while (proxy->shuttingDown == true) {
    // quiet, you'll be dead soon
    pause();
  }
  proxy->shuttingDown = true;
  cout << "Proxy going down." << endl;
  if (proxy->threadCount <= 0) {
    delete proxy->fs;
    exit (0);
  }

  int status =  close (proxy->listenFD);
  if (status < 0) {
    cout << "Close socket call failed:" << status <<  endl;
  }

#ifndef SOLARIS
  timespec brief;
  timespec remainder;
  brief.tv_sec = 1;
  brief.tv_nsec = 0;

  nanosleep (&brief, &remainder);
#endif

  cout << "Hit % = " << ((double)ProxyThread::totalHitCount / (double)ProxyThread::totalRequestCount) << endl;
  cout << "Requests: " << ProxyThread::totalRequestCount << "\nHits: " << ProxyThread::totalHitCount << endl;

  for (int i = 0; i < proxy->threadCount; i++) {
    // cout << "about to test thread " << i << " for null" << endl;
    if (proxy->threads[i] != NULL) {
      delete proxy->threads[i];
      // cout << "destroyed thread " << i << endl;
    }
  }  
  delete proxy->fs;
  exit (0);
}

Proxy::Proxy (int port) {
  cout << "proxy ctor" << endl;
  //  void sig_int (int), thread_make (int);
  //  void proxyExit (int);

  Proxy::globalProxy = this;

  listenFD = -1;
  shuttingDown = false;
  threads = NULL;
  threadCount = 0;

  // we're going to be making a lot of dns lookups so we set this
  //  sethostent (1);

  // init main accept loop mutex
  pthread_mutex_init (&ProxyThread::acceptLock, NULL);
  pthread_mutex_init (&ProxyThread::FSLock, NULL);


// #ifdef USE_CFS
  Status status = OK;
  //  fs = new CFS ("/u/l/e/ledlie/private/thesis/cfs_cache/data", GROUP_COUNT, 1024, status);
  fs = new CFS ("/dev/raw/raw4", GROUP_COUNT, 1024 * 2, status);
  if (status != OK) {
    cout << "Problem creating CFS in Proxy ctor: " << status << endl;
  }
// #else
  //char *base = "/u/l/e/ledlie/private/thesis/ffs_cache";
  //  char *base = "/home/ledlie/proxy/ffs_cache";
//  char *base = "/mnt/scsi_b/data";
//  fs = new FFS (base, status);
//  if (status != OK) {
//    cout << "Problem creating FFS in Proxy ctor: " << status << endl;
//  }


  //#endif

  // note: move this connection code to another fn.  otherwise can't bind results in seg fault

  if ( (listenFD = socket (PF_INET, SOCK_STREAM, 0)) < 0) {
    cerr << "downstream: can't open stream socket" << endl;
    proxyExit (0);
  }

  memset (&clientAddress, 0, sizeof (clientAddress));
  clientAddress.sin_family = AF_INET;
  clientAddress.sin_addr.s_addr = htonl (INADDR_ANY);
  clientAddress.sin_port = htons (port);

  addressLength = sizeof (clientAddress);

  if (bind (listenFD, (struct sockaddr *) &clientAddress, sizeof (clientAddress)) < 0) {
    cerr << "server: can't bind local address" << endl;
    proxyExit (0);
  }

  // why 5?
  
  int listenCode = listen (listenFD, 5);
  if (listenCode < 0) {
    cout << "listen returned  " << listenCode << endl;
    proxyExit (0);
  }

  ProxyThread::totalRequestCount = 0;
  ProxyThread::totalHitCount = 0;
  threadCount = PROXY_THREAD_COUNT;

  //  tptr = (ProxyThread*) calloc (threadCount, sizeof (ProxyThread));
  threads = new ProxyThread* [threadCount];

  for (int i = 0; i < threadCount; i++) {
    void *startThreadLoop (void *);
    ProxyThread *proxyThread = new ProxyThread (i, clientAddress, listenFD, fs);
    threads[i] = proxyThread;

    pthread_create (&proxyThread->thread_tid, NULL, &startThreadLoop, (void *) proxyThread);
  }

  cout << endl;

  // catch ^C
  signal (SIGINT, proxyExit);
  // catch standard kill
  signal (SIGTERM, proxyExit);

}

Proxy::~Proxy () {
  cout << "proxy destructor" << endl;
  

  proxyExit(0);
}

void * startThreadLoop (void * arg) {
  ProxyThread *p = (ProxyThread *) arg;
  p->run ();
  // never reached
  return NULL;
}

// void Proxy::run () {
// #ifndef LIMITED_RUN
//   for ( ;;)
//     pause ();
// #else
//   sleep (LIMITED_RUN);
//   cout << "Timely death." << endl;
// #endif
//   proxyExit (0);

// }

void Proxy::run () {
  for ( ;;) {
    pause ();
    //    sleep (STATS_DURATION);
// #ifdef USE_CFS
//     cout << *fs << endl;
// #endif
  }

}
