/*
 * Copyright (C) 2001  Jonathan T. Ledlie
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "proxyThread.h"

pthread_mutex_t  ProxyThread::acceptLock;
pthread_mutex_t  ProxyThread::FSLock;
long ProxyThread::totalRequestCount;
long ProxyThread::totalHitCount;

// extern void proxyExit (int signo);

ProxyThread::ProxyThread (int myID, struct sockaddr_in myClientAddress, int myListenFD, CFS *myFS) : id (myID), clientAddress (myClientAddress), listenFD (myListenFD), fs (myFS) {

  //ProxyThread::ProxyThread (int myID, struct sockaddr_in myClientAddress, int myListenFD, FFS *myFS) : id (myID), clientAddress (myClientAddress), listenFD (myListenFD), fs (myFS) {


  cout << id <<  "\t";
  requestCount = 0;
  response.data = NULL;
}

ProxyThread::~ProxyThread () {
}

//=========================================================================

void ProxyThread::run () {
  Status status;
  void handleRequest (int);
  socklen_t clientLength;
  int requestCount = 0;

  for (;;) {
    clear ();

    clientLength = sizeof (clientAddress);
    pthread_mutex_lock (&ProxyThread::acceptLock);
    request.socketFD = accept (listenFD, (struct sockaddr *) &clientAddress, &clientLength);
    pthread_mutex_unlock (&ProxyThread::acceptLock);
    if (request.socketFD <= 0) {
      cout << "accept returned " << request.socketFD << endl;
      pthread_exit ((void *)status);
    }

    requestCount++;
    ProxyThread::totalRequestCount++;
    if (requestCount % 1000 == 0) {
      cout << "T" <<  id << "(" << requestCount << ")" << endl;
    }
   
    status = ProxyThread::handleRequest ();

    if (status != OK) {
      close (response.socketFD);
      close (request.socketFD);
      if (isFatal(status)) {
	cerr << "Thread " << id << " exiting due to: " << status << endl;
	pthread_exit ((void *)status);
      } else {
	cerr << "Thread " << id << " had warning level error: " << status << endl;
      }
    }
    // close happens below
  }

}

//=========================================================================

Status ProxyThread::handleRequest () {
  Status status;
  int numberRead;

  if  ( (numberRead = ::read (request.socketFD, request.data, MAX_LINE)) <= 0) {
    close (request.socketFD);
    return READ_SOCKET_ERR;
  }

  status = parseRequestHeader ();

  if (request.cacheable == false) {
    return passThroughRequest ();
  }

  status = fs->lookup (request.url, request.groupNumber);

  if (status == OK) {
    // found
    status = sendCachedCopy ();
    ProxyThread::totalHitCount++;
    return status;
  }  else {
    // not found
    status = addToCache ();
    return status;

  }

  return OK;
}

//=========================================================================

// fill the components of the request struct
 Status ProxyThread::parseRequestHeader () {
  int headerIndex = 0;
  char *current =  request.data;
  char *line;

  int diff = 0;
  while ( ((line = strchr (current, '\n')) != NULL) && (diff != 1)) {
    diff = line - current ;
    memcpy (request.headers[headerIndex], current, diff);
    headerIndex++;
    current = line + 1;
  }

  char *urlStart = strchr (request.headers[0], ' ');
  char *urlEnd  = strchr (urlStart + 1, ' ');
  memcpy (request.url, urlStart, urlEnd - urlStart);

  // test for ? and cgi (should probably do more
  // note that not setting a few structures here
  if ( ((strchr (request.headers[0], '?')) != NULL) || ((strstr (request.headers[0], "cgi")) != NULL)) {
    cout << "found ? or cgi " << endl;
    request.cacheable = false;
    return OK;
  }

  headerIndex = 0;
  while (request.headers[headerIndex][0] != '\0'  && headerIndex < MAX_HEADER_COUNT) {
#ifdef DEBUG_HEADERS
    cout << request.headers[headerIndex] << endl;
#endif
    // expires header
    if ( (strncmp (request.headers[headerIndex], IF_MODIFIED_SINCE, sizeof (IF_MODIFIED_SINCE) - 1)) == 0) {
       char dateString [MAX_HEADER_WIDTH];
       memcpy (dateString, request.headers[headerIndex] + sizeof (IF_MODIFIED_SINCE), sizeof (SAMPLE_DATE) - 1);
      long epoch = str2time (dateString);
      cout << "modified since: " << epoch << endl;
    }

    // pragma header
    if (strcmp (request.headers[headerIndex], PRAGMA_NO_CACHE) == 0) {
      request.cacheable = false;
      cout << "no cache" << endl;
    }

    headerIndex++;
  }

  return OK;
}

 //=========================================================================

 Status ProxyThread::parseResponseHeader () {
  int headerCount =0;
  int headerIndex = 0;
  char *current =  response.headerData;
  char *line;

  int diff = 0;
  while ( ((line = strchr (current, '\n')) != NULL) && (diff != 1)) {
    diff = line - current ;
    response.headerLength = response.headerLength + diff + 1;    
    memcpy (response.headers[headerIndex], current, diff);
#ifdef DEBUG_HEADERS
    cout << response.headers[headerIndex] << endl;
#endif
    headerIndex++;
    current = line + 1;
  }



  headerCount = headerIndex;

  // test to see if server says it's ok to cache
  for (headerIndex = 0; headerIndex < headerCount; headerIndex++) {
#ifdef DEBUG_HEADERS
    cout << response.headers[headerIndex] << endl;
#endif
    if ( (strncmp (response.headers[headerIndex], EXPIRES, sizeof (EXPIRES) - 1)) == 0) {
      //      cout << "found expires:" << response.headers[headerIndex] << endl;
    } else if ( (strncmp (response.headers[headerIndex], CONTENT_LENGTH, sizeof (CONTENT_LENGTH) - 1)) == 0) {
      //      cout << "found content length:" << response.headers[headerIndex] << endl;
      //      memset (contentLengthString, 0, MAX_SHORT_HEADER);
      int numLength = strlen (response.headers[headerIndex]) - sizeof(CONTENT_LENGTH);
      char contentLengthString [numLength];
      int i = 0;
      for (int offset = sizeof (CONTENT_LENGTH); i < numLength - 1; offset++) {
	//	cout << "i: " << i << " " << response.headers[headerIndex][offset] << endl;
	contentLengthString[i] = response.headers[headerIndex][offset];
	i++;
      }
      //      cout << "header length " << strlen (response.headers[headerIndex]) << endl;
      //      cout << "content length const string " << sizeof(CONTENT_LENGTH) << endl;
      //      memcpy (contentLengthString, response.headers[headerIndex] + sizeof (CONTENT_LENGTH) , numLength);
      contentLengthString [numLength - 1] = '\0';

      //      cout << contentLengthString << "Q" << endl;
      response.length = atoi (contentLengthString);

      //      cout << "response.length" << response.length << ", numLength=" << numLength << endl;
      
    }
  }

  if ((response.length + response.headerLength > MAX_OBJ_SIZE) || (response.length < 0)) {
    request.cacheable = false;
  }

  return OK;
}

 //=========================================================================

 Status ProxyThread::passThroughRequest () {
  Status status;

#ifdef DEBUG_FUNCTION_CALLS
  cout << "pass through request" << endl;
#endif

  if ( (status = openServerSocket ()) != OK) {
    return status;
  }

  int amountWritten;
  amountWritten = writen (response.socketFD, request.data, strlen (request.data));
  status = copyServerToClient (NULL);

  return status;
}

 //=========================================================================

 Status ProxyThread::copyServerToClient (char *seekPtr) {
  int totalRead = 0;
  int totalWritten = 0;

  int amountRead;
  // local version of response's data

#ifdef DEBUG_FUNCTION_CALLS
  cout << "top of copyServerToClient" << endl;
#endif

  char responseData [MAX_LINE];
  while ( (amountRead = read (response.socketFD, responseData, MAX_LINE))  > 0) {
    totalRead += amountRead;

    if (seekPtr != NULL) {
      memcpy (seekPtr, responseData, amountRead);
      seekPtr += amountRead;
    }

    int amountWritten;
    if ( (amountWritten = writen (request.socketFD, responseData, amountRead)) < 0) {
      cout << "problem writing back to client:" << amountWritten << endl;
      return WRITE_SOCKET_ERR;
    }
    totalWritten += amountWritten;
  }

  if (close (response.socketFD) < 0) {
    perror ("Failed to close response socket in copyServerToClient:");
    return SOCKET_ERROR;
  }
  if (close (request.socketFD) < 0) {
    perror ("Failed to close request socket in copyServerToClient:");
    return SOCKET_ERROR;
  }

  return OK;
}

 //=========================================================================

 Status ProxyThread::sendCachedCopy () {
  Status status;

#ifdef DEBUG_FUNCTION_CALLS 
  cout << "top of send cached copy" << endl;
#endif
  char *fileData = NULL;
  int fileLength = 0;

  status = fs->read (request.url, request.groupNumber, fileData, fileLength);
  if (status != OK) {
    return status;
  }
  
  if ( writen (request.socketFD, fileData, fileLength) != fileLength) {
    // close the file

    //#ifdef USE_CFS
    status = fs->release (request.url, request.groupNumber);
    //#endif
    delete [] fileData;
    return WRITE_SOCKET_ERR;
  }

  //#ifdef USE_CFS
  status = fs->release (request.url, request.groupNumber);

 //  if (status != OK) return status;  
//   //#else
//   delete [] fileData;
// #endif

  if (close (request.socketFD) < 0) {
    perror ("Failed to close response socket in sendCachedCopy:");
    return SOCKET_ERROR;
  }


  return OK;

}

 //=========================================================================

 Status ProxyThread::addToCache () {
  Status status;

#ifdef DEBUG_FUNCTION_CALLS
  cout << "addToCache" << endl;
#endif

  // open connection to server
  if ( (status = openServerSocket ()) != OK) {
    return status;
  }

  // send request to server
  int amountWritten;
  amountWritten = writen (response.socketFD, request.data, strlen (request.data));

  // get header from server

  //  cout << "socket = " << response.socketFD << endl;
  int headerPlusExtra;
  if  ( (headerPlusExtra  = ::read (response.socketFD, response.headerData, MAX_LINE)) == 0) {
    close (request.socketFD);
    ASSERT (false);
    return READ_SOCKET_ERR;
  }

  ::write (request.socketFD, response.headerData, headerPlusExtra);

  // parse it
  parseResponseHeader ();

  // see if it's cacheable
  if (request.cacheable == true) {
    // if so, malloc a buffer to pile the data into
    //    cout << "cacheable" << endl;
    if (response.length <= 0) {
      cout << "response size not set!" << endl;      
      return copyServerToClient (NULL);
    }
    //    cout << "response.length" << response.length << ", headerLength:" << response.headerLength << endl;
    //    cout << "headerPlusExtra:" << headerPlusExtra << endl;

    response.data = new char [response.length + response.headerLength ];
    memset (response.data, 0, response.length + response.headerLength);
    memcpy (response.data, response.headerData, headerPlusExtra);

    //    cout << response.data << endl;

    copyServerToClient (response.data + headerPlusExtra);

    //    cout << "final response length = " << strlen (response.data) << endl;

    //    cout << "calling create: " << request.url << endl;
    //    pthread_mutex_lock (&ProxyThread::FSLock);
    //    int groupNumber = rand() % GROUP_COUNT;

    int groupNumber = pickGroup (request.url);
    

    status = fs->create (request.url, groupNumber, response.data, response.length + response.headerLength);
    //    pthread_mutex_unlock (&ProxyThread::FSLock);
    if (status != OK) {
      cout << "Create returned:" << status << endl;
    }

    delete [] response.data;
    response.data = NULL;

  } else {
    //    cout << "not cacheable" << endl;

    // if it's not cacheable, just do a passThrough
    int amountWritten = writen (request.socketFD, response.headerData, headerPlusExtra);
    return copyServerToClient (NULL);
  }
  
  return OK;  
}

int ProxyThread::pickGroup (char *url) {

  // cout << url << endl;

  return rand () % GROUP_COUNT;

}
