/*
 * Copyright (C) 2001  Jonathan T. Ledlie
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef PROXYTHREAD_H
#define PROXYTHREAD_H

#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <iostream.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "cfs.h"
#include "proxy.h"
#include "status.h"
//#ifdef USE_CFS
//#else
//#include "ffs.h"
//#endif

#ifndef MAX_LINE
//#define MAX_LINE 4096
#define MAX_LINE 8192
#endif

#define MAX_URL 200
#define MAX_HEADER_WIDTH 200
#define MAX_HEADER_COUNT 15
#define PRAGMA_NO_CACHE "Pragma: no-cache"
#define IF_MODIFIED_SINCE "If-Modified-Since: "
#define EXPIRES "Expires:"
#define CONTENT_LENGTH "Content-Length:"
#define SAMPLE_DATE "Fri, 15 Apr 1994 00:00:00 GMT"
#define MAX_PORT_CHARS 10
#define MAX_SHORT_HEADER 20
#define DO_DATA_COPY true
#define NO_DATA_COPY false

struct ProxyRequest {
  char url [MAX_URL];
  char data [MAX_LINE];
  int groupNumber;
  // char *data;
  int length;
  struct sockaddr_in serverAddress;
  int port;
  bool refresh;
  bool cacheable;
  int socketFD;
  char headers [MAX_HEADER_COUNT][MAX_HEADER_WIDTH];
};

struct ProxyResponse {
  char *data;
  char headerData [MAX_LINE];
  int headerLength;
  int length;
  int socketFD;
  char headers [MAX_HEADER_COUNT][MAX_HEADER_WIDTH];
};

class ProxyThread {
  friend ostream& operator << (ostream& os, ProxyThread& proxyThread);
  friend ostream& operator << (ostream& os, Status& status);
 public:
  //#ifdef USE_CFS
  ProxyThread (int myID, struct sockaddr_in myClientAddress, int myListenFD, CFS *myFS);
  //#else
//  ProxyThread (int myID, struct sockaddr_in myClientAddress, int myListenFD, FFS *myFS);
  //#endif
  ~ProxyThread ();
  void run ();
  pthread_t thread_tid;
  static pthread_mutex_t acceptLock;
  static pthread_mutex_t FSLock;

  static long totalRequestCount;
  static long totalHitCount;
 private: 
  int pickGroup (char *url);
   Status handleRequest ();
   Status parseRequestHeader ();
   Status parseResponseHeader ();
   Status passThroughRequest ();
   Status openServerSocket ();
   Status sendCachedCopy ();
    Status copyServerToClient (char *seekPtr);
   Status addToCache ();
   ssize_t writen(int fd, void *vptr, ssize_t n);
    ssize_t readn(int fd, void *vptr, ssize_t maxlen);
   long str2time (char *timeString);
  //ssize_t readn(int fd, void *vptr, ssize_t maxlen);
  //  ssize_t writen(int fd, void *vptr, ssize_t n);
  void clear ();
  long requestCount;
  const int id;
  struct ProxyRequest request;
  struct ProxyResponse response;

  struct sockaddr_in clientAddress;
  int listenFD;
  
  //#ifdef USE_CFS
  CFS *fs;
  //#else
//  FFS *fs;
  //#endif

};

//extern void proxyExit (int signo);

#endif
