/*
 * Copyright (C) 2001  Jonathan T. Ledlie
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "bufferManager.h"
#include "request.h"
#include "disk.h"

Request::Request (RequestType a, Page *d, unsigned int frameNum) : frameNumber (frameNum), action (a), data (d) {
  //  ASSERT (frameNumber >= 0);
  ASSERT (frameNumber >= 0 && frameNumber <= Disk::diskFrameCount);
  ASSERT (data != NULL);

  ASSERT (data >=  BufMgr::lowFrameAddress && data <=  BufMgr::highFrameAddress);


  complete = false;
  next = NULL;
  previous = NULL;
  pthread_mutex_init (&mutex, NULL);
  pthread_cond_init (&cv, NULL);

}

Request::~Request () {
  pthread_mutex_destroy (&mutex);
  pthread_cond_destroy (&cv);
}

Status Request::verify () {
  //  cout << "entering verifying request" << endl;
  ASSERT (action == READ || (action == WRITE));
  ASSERT (frameNumber >= 0 && frameNumber < Disk::diskFrameCount);
  ASSERT (data != NULL);
  ASSERT (data >= BufMgr::lowFrameAddress && data <=  BufMgr::highFrameAddress);
  ASSERT (this->previous->next == this);
  ASSERT (this->next->previous == this);
  //  cout << "leaving verifying request" << endl;
  return OK;
}

ostream& operator << (ostream& os, Request* &request) {
  if (request->action == WRITE) 
    os << "w ";
  else if (request->action == READ) 
    os << "r ";
  else 
    os << "? ";

   if (request->complete)
     os << "\tdone";
   else
     os << "\tnot done";

   os << " frame num: " << request->frameNumber;
  return os;
}

