/*
 * Copyright (C) 2001  Jonathan T. Ledlie
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <set>
#include <list>
#include <iostream>

class Reader {
  friend ostream& operator << (ostream& os, const Reader& reader);
public:
  int file;
  int pageNo;

  Reader (int f, int p) : file (f), pageNo (p) { }
  Reader (const Reader& r) : file (r.file), pageNo (r.pageNo) { }
  ~Reader() { }
  Reader& operator= (const Reader& r) { file = r.file; pageNo = r.pageNo; }

  bool operator==(const Reader& r) const {  return ( (this->file == r.file) && (this->pageNo == r.pageNo)); }
  bool operator< (const Reader& r) const { return ( (this->file < r.file) && (this->pageNo < r.pageNo) ); }
};

int main () {

  set<Reader> reader;

  Reader r1 (1, 1);
  Reader r2 (2, 2);
  Reader r3 (3, 3);
  //  r1.file = 1;

  //  reader.push_back (r1);
  reader.insert (r1);
  reader.insert (r3);
  //  reader.insert (r2);

  //  cout << reader << endl;
  cout << reader.size() << endl;


  cout << "Reader: ";
  for (set<Reader>::const_iterator iter = reader.begin(); iter != reader.end(); iter++) {
    cout << *iter << " ";
  }
  cout << endl;

  set<Reader>::const_iterator it = reader.find(r2);

  if (it == reader.end()) {
    cout << "not found" << endl;
  } else {
    cout << "found" << endl;
    reader.erase (it);
  }

  // reader.remove (it);

  cout << reader.size() << endl;

  return 0;
}

ostream& operator << (ostream& os, const Reader &reader) {
  os << "(" << reader.file << "," << reader.pageNo << ")";
  return os;
}
