/*
 * Copyright (C) 2001  Jonathan T. Ledlie
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef STATUS_H
#define STATUS_H

#include <iostream>
#include <unistd.h>

//#define SHOWFILES
//#define DEBUG_HEADERS
//#define DEBUG_API
//#define DEBUG_NETWORK
//#define DEBUG_FUNCTION_CALLS
/* #define DEBUG_LOOKUP */
//#define DEBUG_CREATE
/* #define DEBUG_READ */
//#define DEBUG_DELETE
//#define DEBUG_GROUP
//#define DEBUG_FILETABLE
//#define DEBUGBUF
//#define DEBUG_READLIST
//#define DEBUG_BUFHASHTBL
//#define DEBUG_DISK
//#define SLOW_DISK 3
//#define DEBUG_FILE
//#define DEBUG_QUEUE
//#define DEBUG_ASSERT

#ifdef DEBUG_ASSERT
#define ASSERT(c)  {                                                         \
                     if (!(c)) {                                             \
                        cerr <<                                      \
                                "ASSERTION FAILURE: " <<                \
                                __FILE__ <<                                    \
                                __LINE__ << "\nFailed condition: \n" << #c <<  endl;     \
                        pause();                                            \
                     }                                                       \
                   }
#define ASSERTSTATUS(c)  {                                                         \
                     if (!(c)) {                                             \
                        cerr <<                                      \
                                "ASSERTION FAILURE: " <<                \
                                __FILE__ <<                                    \
                                __LINE__ << "\nFailed condition: \n" << #c <<  "\n" << status << endl;     \
                        pause();                                             \
                     }                                                       \
                   }
#else
#define ASSERT(c)
#define ASSERTSTATUS(c)
#endif

enum Status {
    OK = 0,

    NULL_STATUS,
    
    // connection problems
    READ_SOCKET_ERR,
    WRITE_SOCKET_ERR,
    NAME_LOOKUP_ERROR,
    SERVER_CONNECT_ERROR,
    SOCKET_ERROR,

    // io problems
    IO_ERROR,
    FILE_NOT_FOUND,

    // memory problems
    OUT_OF_MEMORY,

    // buffer manager
    NOSPACE,
    NORECORDS,
    INVALIDSLOTNO,
    ENDOFPAGE,
    BADPAGENO,
    BADPAGEPTR,
    BUFFEREXCEEDED,
    HASHNOTFOUND,
    BADBUFFER,
    PAGENOTPINNED,
    PAGEPINNED,
    BUFHASHTBLERROR,
    READ_LIST_ERROR,
    IN_READ_QUEUE,

    // db errors
    FILEEXISTS,
    UNIXERR,
    FILENOTOPEN,
    BADFILE,
    FILEOPEN,
    BADFILEPTR,

    // disk errors
    REQUEST_QUEUE_ERROR,
    DISK_WRITE_ERROR,
    DISK_READ_ERROR,
    DISK_ERROR,
    DISK_FULL,

    // group and file table errors
    NULL_PAGE,
    NEW_BUFFER,
    PAGES_IN_MEMORY_VECTOR_ERROR

};

ostream& operator << (ostream& os, Status& status);
bool isFatal (const Status &status);

#endif
