/*
 * Copyright (C) 2001  Jonathan T. Ledlie
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include <iostream>
#include "bufferManager.h"

   // cout << s << endl;

#define CALL(c)    { Status s; \
                     if ((s = c) != OK) { \
		       cerr << "At line " << __LINE__ << ":" << s << endl << "  "; \
                                           exit(1); \
                     } \
                   }

#define FAIL(c)  { Status s; \
                   if ((s = c) == OK) { \
                     cerr << "At line " << __LINE__ << ":" << endl << "  "; \
                     cerr << "This call should fail: " #c << endl; \
                     exit(1); \
		     } \
		     }

BufMgr*     bufMgr;

#define LOVE 1
#define HATE 0

int main()
{
  //    Error       error;
  
    DB          db;
    File*	file1;
    File*	file2;
    File* 	file3;
    File*       file4;
    int		i;
    const int   num = 100;
    int         j[num];    

    // create buffer manager

    bufMgr = new BufMgr(num);

    // create dummy files

    (void)db.destroyFile("test.1");
    (void)db.destroyFile("test.2");
    (void)db.destroyFile("test.3");
    (void)db.destroyFile("test.4");

    CALL(db.createFile("test.1"));
    ASSERT(db.createFile("test.1") == FILEEXISTS);
    CALL(db.createFile("test.2"));
    CALL(db.createFile("test.3"));
    CALL(db.createFile("test.4"));

    CALL(db.openFile("test.1", file1));
    CALL(db.openFile("test.2", file2));
    CALL(db.openFile("test.3", file3));
    CALL(db.openFile("test.4", file4));

    // test buffer manager

    Page* page;
    Page* page2;
    Page* page3;
    Page cmp;
    int pageno, pageno2, pageno3;

    cout << "Allocating pages in a file..." << endl;
    for (i = 0; i < num; i++) {
      CALL(bufMgr->allocPage(file1, j[i], page));
      sprintf((char*)page, "test.1 Page %d %7.1f", j[i], (float)j[i]);
      CALL(bufMgr->unPinPage(file1, j[i], true, LOVE));
    }

    cout << "Reading pages back..." << endl;
    for (i = 0; i < num; i++) {
      CALL(bufMgr->readPage(file1, j[i], page));
      sprintf((char*)&cmp, "test.1 Page %d %7.1f", j[i], (float)j[i]);
      ASSERT(memcmp(page, &cmp, strlen((char*)&cmp)) == 0);
      CALL(bufMgr->unPinPage(file1, j[i], false, HATE));
    }

    cout << "Writing and reading back multiple files..." << endl;
    for (i = 0; i < num/3; i++) 
    {
      CALL(bufMgr->allocPage(file2, pageno2, page2));
      sprintf((char*)page2, "test.2 Page %d %7.1f", pageno2, (float)pageno2);
      CALL(bufMgr->allocPage(file3, pageno3, page3));
      sprintf((char*)page3, "test.3 Page %d %7.1f", pageno3, (float)pageno3);
      pageno = j[random() % num];
      CALL(bufMgr->readPage(file1, pageno, page));
      sprintf((char*)&cmp, "test.1 Page %d %7.1f", pageno, (float)pageno);
      ASSERT(memcmp(page, &cmp, strlen((char*)&cmp)) == 0);
      cout << (char*)page << endl;
      CALL(bufMgr->readPage(file2, pageno2, page2));
      sprintf((char*)&cmp, "test.2 Page %d %7.1f", pageno2, (float)pageno2);
      ASSERT(memcmp(page2, &cmp, strlen((char*)&cmp)) == 0);
      CALL(bufMgr->readPage(file3, pageno3, page3));
      sprintf((char*)&cmp, "test.3 Page %d %7.1f", pageno3, (float)pageno3);
      ASSERT(memcmp(page3, &cmp, strlen((char*)&cmp)) == 0);
      CALL(bufMgr->unPinPage(file1, pageno, true, HATE));
    }

    for (i = 0; i < num/3; i++) {
      CALL(bufMgr->unPinPage(file2, i+1, true, LOVE));
      CALL(bufMgr->unPinPage(file2, i+1, true, HATE));
      CALL(bufMgr->unPinPage(file3, i+1, true, LOVE));
      CALL(bufMgr->unPinPage(file3, i+1, true, HATE));
    }

#ifdef DEBUGBUF
    bufMgr->printSelf();
#endif DEBUGBUF

    cout << "\nReading \"test.1\"...\n";
    for (i = 1; i < num/3; i++) {
      CALL(bufMgr->readPage(file1, i, page2));
      sprintf((char*)&cmp, "test.1 Page %d %7.1f", i, (float)i);
      ASSERT(memcmp(page2, &cmp, strlen((char*)&cmp)) == 0);
      CALL(bufMgr->unPinPage(file1, i, false, HATE));
    }

    cout << "\nReading \"test.2\"...\n";
    for (i = 1; i < num/3; i++) {
      CALL(bufMgr->readPage(file2, i, page2));
      sprintf((char*)&cmp, "test.2 Page %d %7.1f", i, (float)i);
      ASSERT(memcmp(page2, &cmp, strlen((char*)&cmp)) == 0);
      cout << (char*)page2 << endl;
      CALL(bufMgr->unPinPage(file2, i, false, LOVE));
    }

    cout << "\nReading \"test.3\"...\n";
    for (i = 1; i < num/3; i++) {
      CALL(bufMgr->readPage(file3, i, page3));
      sprintf((char*)&cmp, "test.3 Page %d %7.1f", i, (float)i);
      ASSERT(memcmp(page3, &cmp, strlen((char*)&cmp)) == 0);
      cout << (char*)page3 << endl;
      CALL(bufMgr->unPinPage(file3, i, false, HATE));
    }

    cout << "\nTesting error condition...\n";
    cout << "Following tests should return error codes \n";

    Status status;
    FAIL(status = bufMgr->readPage(file4, 1, page));
    cout << status << endl;
    CALL(bufMgr->allocPage(file4, i, page));
    CALL(bufMgr->unPinPage(file4, i, true,false));
    FAIL(status = bufMgr->unPinPage(file4, i, false, LOVE));
    cout << status << endl;
    for (i = 0; i < num; i++) {
      CALL(bufMgr->allocPage(file4, j[i], page));
      sprintf((char*)page, "test.4 Page %d %7.1f", j[i], (float)j[i]);
    }

    int tmp;
    FAIL(status = bufMgr->allocPage(file4, tmp, page));
    cout << status << endl;

#ifdef DEBUGBUF
    bufMgr->printSelf();
#endif DEBUGBUF

    cout << "\n\nDone with error testing\n";
    for (i = 0; i < num; i++)
      CALL(bufMgr->unPinPage(file4, i+2, true, HATE));
    
    cout << "\nReading \"test.1\"...\n";
    for (i = 1; i < num; i++) {
      CALL(bufMgr->readPage(file1, i, page));
      sprintf((char*)&cmp, "test.1 Page %d %7.1f", i, (float)i);
      ASSERT(memcmp(page, &cmp, strlen((char*)&cmp)) == 0);
      cout << (char*)page << endl;
    }
    
    cout << "flushing file with pages still pinned. Should generate an error" << endl;
    FAIL(status = bufMgr->flushFile(file1));
    cout << status << endl;

    for (i = 1; i < num; i++) 
      CALL(bufMgr->unPinPage(file1, i, true, HATE));

    CALL(bufMgr->flushFile(file1));


    CALL(db.closeFile(file1));
    CALL(db.closeFile(file2));
    CALL(db.closeFile(file3));
    CALL(db.closeFile(file4));

    CALL(db.destroyFile("test.1"));
    CALL(db.destroyFile("test.2"));
    CALL(db.destroyFile("test.3"));
    CALL(db.destroyFile("test.4"));

    delete bufMgr;

    cout << endl << "Passed all tests." << endl;

    return (1);
}
